.alertify, .alertify-show, .alertify-log {
    -webkit-transition: all 500ms cubic-bezier(0.175, 0.885, 0.32, 1.275);
    transition: all 500ms cubic-bezier(0.175, 0.885, 0.32, 1.275);
    /* easeOutBack */
}

.alertify-hide {
    -webkit-transition: all 250ms cubic-bezier(0.6, -0.28, 0.735, 0.045);
    transition: all 250ms cubic-bezier(0.6, -0.28, 0.735, 0.045);
    /* easeInBack */
}

.alertify-log-hide {
    -webkit-transition: all 500ms cubic-bezier(0.6, -0.28, 0.735, 0.045);
    transition: all 500ms cubic-bezier(0.6, -0.28, 0.735, 0.045);
    /* easeInBack */
}

.alertify-cover {
    position: fixed;
    z-index: 99999;
    top: 0;
    right: 0;
    bottom: 0;
    left: 0;
    background-color: white;
    filter: alpha(opacity=0);
    opacity: 0;
}

.alertify-cover-hidden {
    display: none;
}

.alertify {
    position: fixed;
    z-index: 99999;
    top: 50px;
    left: 50%;
    width: 550px;
    margin-left: -275px;
    opacity: 1;
}

.alertify-hidden {
    opacity: 0;
    display: none;
}

/* overwrite display: none; for everything except IE6-8 */
:root * > .alertify-hidden {
    display: block;
    visibility: hidden;
}

.alertify-logs {
    position: fixed;
    z-index: 5000;
    bottom: 10px;
    right: 10px;
    width: 300px;
}

.alertify-logs-hidden {
    display: none;
}

.alertify-log {
    display: block;
    margin-top: 10px;
    position: relative;
    right: -300px;
    opacity: 0;
}

.alertify-log-show {
    right: 0;
    opacity: 1;
}

.alertify-log-hide {
    -webkit-transform: translate(300px, 0);
    -ms-transform: translate(300px, 0);
    transform: translate(300px, 0);
    opacity: 0;
}

.alertify-dialog {
    padding: 25px;
}

.alertify-resetFocus {
    border: 0;
    clip: rect(0 0 0 0);
    height: 1px;
    margin: -1px;
    overflow: hidden;
    padding: 0;
    position: absolute;
    width: 1px;
}

.alertify-inner {
    text-align: center;
}

.alertify-text {
    margin-bottom: 15px;
    width: 100%;
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
    font-size: 100%;
}

.alertify-button, .alertify-button:hover, .alertify-button:active, .alertify-button:visited {
    background: none;
    text-decoration: none;
    border: none;
    /* line-height and font-size for input button */
    line-height: 1.5;
    font-size: 100%;
    display: inline-block;
    cursor: pointer;
    margin-left: 5px;
}

@media only screen and (max-width: 680px) {
    .alertify, .alertify-logs {
        width: 90%;
        -webkit-box-sizing: border-box;
        -moz-box-sizing: border-box;
        box-sizing: border-box;
    }

    .alertify {
        left: 5%;
        margin: 0;
    }
}

/**
 * Twitter Bootstrap Look and Feel
 * Based on http://twitter.github.com/bootstrap/
 */
.alertify, .alertify-log {
    font-family: sans-serif;
}

.alertify {
    background: #FFF;
    border: 1px solid #8E8E8E;
    /* browsers that don't support rgba */
    border: 1px solid rgba(0, 0, 0, 0.3);
    border-radius: 6px;
    -webkit-box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
    box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
    -webkit-background-clip: padding-box;
    background-clip: padding-box;
}

.alertify-dialog {
    padding: 0;
}

.alertify-inner {
    text-align: left;
}

.alertify-message {
    padding: 15px;
    margin: 0;
    text-align: center;
}

.alertify-text-wrapper {
    padding: 0 15px;
}

.alertify-text {
    color: #555;
    border-radius: 4px;
    padding: 8px;
    background-color: #FFF;
    border: 1px solid #CCC;
    -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
    box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
}

.alertify-text:focus {
    border-color: rgba(82, 168, 236, 0.8);
    outline: 0;
    -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(82, 168, 236, 0.6);
    box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(82, 168, 236, 0.6);
}

.alertify-buttons {
    padding: 14px 15px 15px;
    background: #F5F5F5;
    border-top: 1px solid #DDD;
    border-radius: 0 0 6px 6px;
    -webkit-box-shadow: inset 0 1px 0 #FFF;
    box-shadow: inset 0 1px 0 #FFF;
    text-align: right;
}

.alertify-button, .alertify-button:hover, .alertify-button:focus, .alertify-button:active {
    margin-left: 10px;
    border-radius: 4px;
    font-weight: normal;
    padding: 4px 12px;
    text-decoration: none;
    -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
    box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
    background-image: -webkit-gradient(linear, left top, left bottom, from(rgba(255, 255, 255, 0.3)), to(rgba(255, 255, 255, 0)));
    background-image: -webkit-linear-gradient(top, rgba(255, 255, 255, 0.3), rgba(255, 255, 255, 0));
    background-image: linear-gradient(top, rgba(255, 255, 255, 0.3), rgba(255, 255, 255, 0));
}

.alertify-button:focus {
    outline: none;
    -webkit-box-shadow: 0 0 5px #2B72D5;
    box-shadow: 0 0 5px #2B72D5;
}

.alertify-button:active {
    position: relative;
    -webkit-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
    box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
}

.alertify-button-cancel, .alertify-button-cancel:hover, .alertify-button-cancel:focus, .alertify-button-cancel:active {
    text-shadow: 0 -1px 0 rgba(255, 255, 255, 0.75);
    background-color: #E6E6E6;
    border: 1px solid #BBB;
    color: #333;
    background-image: -webkit-gradient(linear, left top, left bottom, from(#FFF), to(#E6E6E6));
    background-image: -webkit-linear-gradient(top, #FFF, #E6E6E6);
    background-image: linear-gradient(top, #FFF, #E6E6E6);
}

.alertify-button-cancel:hover, .alertify-button-cancel:focus, .alertify-button-cancel:active {
    background: #E6E6E6;
}

.alertify-button-ok, .alertify-button-ok:hover, .alertify-button-ok:focus, .alertify-button-ok:active {
    text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
    background-color: #04C;
    border: 1px solid #04C;
    border-color: #04C #04C #002A80;
    border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
    color: #FFF;
}

.alertify-button-ok:hover, .alertify-button-ok:focus, .alertify-button-ok:active {
    background: #04C;
}

.alertify-log {
    background: #D9EDF7;
    padding: 8px 14px;
    border-radius: 4px;
    color: #3A8ABF;
    text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
    border: 1px solid #BCE8F1;
}

.alertify-log-error {
    color: #B94A48;
    background: #F2DEDE;
    border: 1px solid #EED3D7;
}

.alertify-log-success {
    color: #468847;
    background: #DFF0D8;
    border: 1px solid #D6E9C6;
}
