html,
body {
    margin: 0;
    padding: 0;
    font-family: "Lucida Sans Unicode", "Lucida Grande", sans-serif;
}

body {
    color: #006;
    padding: 10px;
}

div#footer {
    clear: both;
    background: #333;
    color: #FFF;
    width: 100%;
}

div#footer a {
    display: inline;
    padding: 0;
    color: #C6D5FD;
    text-align: right;
}

/*
 * Table that host data rows on all pages
 */
.one-column-emphasis {
    width: 100% !important;
    font-size: 12px;
    text-align: left;
    border-collapse: collapse;
}

.one-column-emphasis th {
    font-size: 14px;
    font-weight: normal;
    padding: 12px 15px;
    color: #039;
}

.one-column-emphasis td {
    padding: 10px 15px;
    color: #669;
    border-top: 1px solid #e8edff;
}

.one-column-emphasis tr:hover td {
    color: #339;
    background: #eff2ff;
}

.one-column-emphasis-first {
    background: #d0dafd;
    border-right: 10px solid transparent;
    border-left: 10px solid transparent;
}

/*
 * Table that holds list of tabs on top of the page
 */
.header-table {
    font-size: 12px;
    width: 100%;
    min-width: 1200px;
    table-layout: fixed;
    text-align: left;
    border-collapse: collapse;
    color: #FFFFFF;
    background: #333333;
}

.header-table th {
    font-size: 20px;
    font-weight: bold;
    padding: 12px 15px;
}

.header-table td {
    padding: 10px 15px;
    border-top: 1px solid #FFFFFF;
}

.header-table tr td a {
    text-decoration: none;
    color: #FFFFFF;
}

.header-table tr:hover td:hover {
    background: lightsalmon;
}

.header-table td.active {
    background: #CC0000;
}

.header-table tr:hover td:hover a, .header-table tr:hover td:hover a:hover {
    text-decoration: none;
    color: black;
    background: lightsalmon;
}

.side-tool-bar {
    float: right;
    padding: 5px;
    font-size: 12px;
}

/* highlighting rows in dataTables accordingly to Job state */
tr.state_embryo {
    background-color: #D5F5FF !important;
}

tr.state_embryo td {
    color: #000000 !important;
}

tr.state_in_progress {
    background-color: #C3FDB8 !important;
}

tr.state_in_progress td {
    color: #000000 !important;
}

tr.state_processed {
    background-color: #15C200 !important;
}

tr.state_processed td {
    color: #000000 !important;
}

tr.state_final_run {
    background-color: #ADFF2F !important;
}

tr.state_final_run td {
    color: #000000 !important;
}

tr.state_skipped {
    background-color: #FF4500 !important;
}

tr.state_skipped td {
    color: #000000 !important;
}

tr.state_noop {
    background-color: #FFFFAA !important;
}

tr.state_noop td {
    color: #000000 !important;
}

tr.state_inconsistent {
    background-color: #d28aff !important;
}

tr.state_inactive {
    background-color: lightgray !important;
}
