<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
    <meta http-equiv="content-type" content="text/html; charset=UTF-8"/>
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', file='css/mx_page.css') }}">
    <link rel="stylesheet" type="text/css" href="{{ url_for('static', file='css/font-awesome-4.3.0.css') }}">
    <script type="text/javascript" src="{{ url_for('static', file='js/mx_utils.js') }}"></script>
    <script type="text/javascript" src="{{ url_for('static', file='js/jquery-2.1.4.min.js') }}"></script>
    <title>Object Viewer</title>
</head>

<body>
<pre id="json"></pre>

<form style="text-align: center">
    <input type="button" class="action_button fa-input" value="&#xf00d;&nbsp;Close" onclick="window.close()"/>
</form>

<script type="text/javascript">
    $(document).ready(function () {
        var action = get_url_parameter('action');
        var url = '/' + action + '/?' + window.location.search.substring(1);

        $.get(url, function (data) {
            $("#json").text(formatJSON(data));
        });
    });
</script>

</body>
</html>
